@@ -7,7 +7,6 @@ authorization is required to interact with private documents.
 
 Further documentation:
 
-    http://datadesk.github.io/python-documentcloud/
     https://www.documentcloud.org/help/api
 
 """
@@ -33,13 +32,12 @@ class BaseDocumentCloudClient(object):
     """
     Patterns common to all of the different API methods.
     """
-    BASE_URI = 'https://www.documentcloud.org/api/'
+    BASE_URI = u'https://www.documentcloud.org/api/'
     
     def __init__(self, username, password):
         self.username = username
         self.password = password
     
-    @retry(Exception, tries=4)
     def _make_request(self, url, params=None, opener=None):
         """
         Configure a HTTP request, fire it off and return the response.
@@ -86,23 +84,14 @@ class BaseDocumentCloudClient(object):
             # Pull the document_ids out of the params
             document_ids = params.get("document_ids")
             del params['document_ids']
-            params = urllib.urlencode(params, doseq=True)
+            params = urllib.urlencode(params)
             # These need to be specially formatted in the style documentcloud
             # expects arrays. The example they provide is:
             # ?document_ids[]=28-boumediene&document_ids[]=207-academy&document_ids[]=30-insider-trading
             params += "".join(['&document_ids[]=%s' % id for id in document_ids])
-        # More special case handler of key/value data tags, if they exist
-        elif params.get("data", None):
-            # Pull them out of the dict
-            data = params.get("data")
-            del params['data']
-            params = urllib.urlencode(params, doseq=True)
-            # Format them in the style documentcloud expects
-            # ?data['foo']=bar&data['tit']=tat
-            params += "".join(['&data[%s]=%s' % (key, value) for key, value in data.items()])
         else:
             # Otherwise, we can just use the vanilla urllib prep method
-            params = urllib.urlencode(params, doseq=True)
+            params = urllib.urlencode(params)
         # Make the request
         content = self._make_request(
             self.BASE_URI + method,
@@ -115,7 +104,7 @@ class BaseDocumentCloudClient(object):
         """
         # Encode params if they exist
         if params:
-            params = urllib.urlencode(params, doseq=True)
+            params = urllib.urlencode(params)
         content = self._make_request(
             self.BASE_URI + method,
             params,
@@ -157,7 +146,7 @@ class DocumentClient(BaseDocumentCloudClient):
             'per_page': per_page,
             'mentions': 3,
         }
-        data = self.fetch('search.json', params)
+        data = self.fetch(u'search.json', params)
         return data.get("documents")
     
     def search(self, query):
@@ -193,7 +182,7 @@ class DocumentClient(BaseDocumentCloudClient):
         
         Example usage:
         
-            >> documentcloud.documents.get('71072-oir-final-report')
+            >> documentcloud.documents.get(u'71072-oir-final-report')
         
         """
         data = self.fetch('documents/%s.json' % id).get("document")
@@ -201,85 +190,37 @@ class DocumentClient(BaseDocumentCloudClient):
         return Document(data)
     
     @credentials_required
-    def upload(self, pdf, title=None, source=None, description=None,
-        related_article=None, published_url=None, access='private',
-        project=None, data=None, secure=False):
+    def upload(self, path, title=None, source=None,description=None,
+        related_article=None, published_url=None,
+        access='private', project=None):
         """
         Upload a PDF or other image file to DocumentCloud.
         
-        You can submit either a pdf opened as a file object or a path to a pdf file.
-        
         Example usage:
         
-            # From a file path
-            >> documentcloud.documents.upload("/home/ben/sample.pdf", "sample title")
+            >> documentcloud.documents.upload("/home/ben/sample.pdf", "sample")
         
-            # From a file object
-            >> pdf = open(path, 'rb')
-            >> documentcloud.documents.upload(pdf, "sample title")
-
-        Returns the document that's created as a Document object.
+        Returns the documentcloud id of the document that's created.
         
         Based on code developed by Mitchell Kotler and refined by Christopher Groskopf.
         """
         # Required parameters
-        if hasattr(pdf, 'read'):
-            params = {'file': pdf}
-        else:
-            params = {'file': open(pdf, 'rb')}
+        params = {'file': open(path, 'rb')}
         # Optional parameters
         if title:
             params['title'] = title
         else:
             # Set it to the file name
-            if hasattr(pdf, 'read'):
-                params['title'] = pdf.name.split(os.sep)[-1].split(".")[0]
-            else:
-                params['title'] = pdf.split(os.sep)[-1].split(".")[0]
+            params['title'] = path.split(os.sep)[-1].split(".")[0]
         if source: params['source'] = source
         if description: params['description'] = description
         if related_article: params['related_article'] = related_article
         if published_url: params['published_url'] = published_url
         if access: params['access'] = access
         if project: params['project'] = project
-        if data:
-            for key, value in data.items():
-                is_valid_data_keyword(key)
-                params['data[%s]' % key] = value
-        if secure: params['secure'] = 'true'
         # Make the request
         response = self._make_request(self.BASE_URI + 'upload.json', params, MultipartPostHandler)
-        return self.get(json.loads(response)['id'])
-    
-    @credentials_required
-    def upload_directory(self, path, source=None, description=None,
-        related_article=None, published_url=None, access='private',
-        project=None, data=None, secure=False):
-        """
-        Uploads all the PDFs in the provided directory.
-        
-        Example usage:
-        
-            >> documentcloud.documents.upload_directory("/home/ben/pdfs/")
-        
-        Returns a list of the documents created during the upload.
-        
-        Based on code developed by Mitchell Kotler and refined by Christopher Groskopf.
-        """
-        # Loop through the path and get all the files
-        path_list = []
-        for (dirpath, dirname, filenames) in os.walk(path):
-            path_list.extend([os.path.join(dirpath, i) for i in filenames
-                if i.lower().endswith(".pdf")])
-        # Upload all the pdfs
-        obj_list = []
-        for pdf_path in path_list:
-            obj = self.upload(pdf_path, source=source, description=description,
-                related_article=related_article, published_url=published_url,
-                access=access, project=project, data=data, secure=secure)
-            obj_list.append(obj)
-        # Pass back the list of documents
-        return obj_list
+        return json.loads(response)['id']
     
     @credentials_required
     def delete(self, id):
@@ -322,49 +263,20 @@ class ProjectClient(BaseDocumentCloudClient):
             obj_list.append(proj)
         return obj_list
     
-    def get(self, id=None, title=None):
+    def get(self, id):
         """
-        Retrieve a particular project using its unique identifier or it's title.
-        
-        But not both.
+        Retrieve a particular project using its unique identifier.
         
         Example usage:
         
-            >> documentcloud.projects.get('arizona-shootings')
+            >> documentcloud.projects.get(u'arizona-shootings')
         
         """
-        # Make sure the kwargs are kosher
-        if id and title:
-            raise ValueError("You can only retrieve a Project by id or title, not by both")
-        elif not id and not title:
-            raise ValueError("You must provide an id or a title to make a request.")
-        # Pull the hits
-        if id:
-            hit_list = [i for i in self.all() if str(i.id) == str(id)]
-        elif title:
-            hit_list = [i for i in self.all() if i.title.lower().strip() == title.lower().strip()]
-        # Throw an error if there's more than one hit.
-        if len(hit_list) > 1:
-            raise DuplicateObjectError("There is more than one project that matches your request.")
-        # Try to pull the first hit
         try:
-            return hit_list[0]
+            return [i for i in self.all() if str(i.id) == str(id)][0]
         except IndexError:
-            # If it's not there, you know to throw this error.
             raise DoesNotExistError("The resource you've requested does not exist or is unavailable without the proper credentials.")
     
-    def get_by_id(self, id):
-        """
-         A reader-friendly shortcut to retrieve projects based on their ID. 
-        """
-        return self.get(id=id)
-    
-    def get_by_title(self, title):
-        """
-        A reader-friendly shortcut to retrieve projects based on their title. 
-        """
-        return self.get(title=title)
-    
     @credentials_required
     def create(self, title, description=None, document_ids=None):
         """
@@ -381,7 +293,7 @@ class ProjectClient(BaseDocumentCloudClient):
             'title': title,
         }
         if description: params['description'] = description
-        params = urllib.urlencode(params, doseq=True)
+        params = urllib.urlencode(params)
         if document_ids:
             # These need to be specially formatted in the style documentcloud
             # expects arrays. The example they provide is:
@@ -392,24 +304,7 @@ class ProjectClient(BaseDocumentCloudClient):
         # If it doesn't exist, that suggests the project already exists
         if not new_id:
             raise DuplicateObjectError("The Project title you tried to create already exists")
-        # Fetch the actual project object from the API and return that.
-        return self.get(new_id)
-    
-    @credentials_required
-    def get_or_create_by_title(self, title):
-        """
-        Fetch a title, if it exists. Create it if it doesn't.
-        
-        Returns a tuple with the object first, and then a boolean that indicates
-        whether or not the object was created fresh. True means it's brand new.
-        """
-        try:
-            obj = self.get_by_title(title)
-            created = False
-        except DoesNotExistError:
-            obj = self.create(title=title)
-            created = True
-        return obj, created
+        return new_id
     
     @credentials_required
     def delete(self, id):
@@ -497,27 +392,19 @@ class Document(BaseAPIObject):
             * related_article
             * access
             * published_url
-            * data key/value pairs
         
         Returns nothing.
         """
         params = dict(
-            title=self.title or '',
-            source=self.source or '',
-            description=self.description or '',
-            related_article=self.resources.related_article or '',
-            published_url=self.resources.published_url or '',
+            title=self.title,
+            source=self.source,
+            description=self.description,
+            related_article=self.resources.related_article,
+            published_url=self.resources.published_url,
             access=self.access,
-            data=self.data,
         )
         self._connection.put('documents/%s.json' % self.id, params)
     
-    def save(self):
-        """
-        An alias to `'put` that post changes back to DocumentCloud
-        """
-        self.put()
-
     def delete(self):
         """
         Deletes this object from documentcloud.org.
@@ -528,30 +415,16 @@ class Document(BaseAPIObject):
     # Lazy loaded attributes
     #
     
-    def _lazy_load(self):
-        """
-        Fetch metadata if it was overlooked during the object's creation.
-        
-        This can happen when you retrieve documents via search, because
-        the JSON response does not include complete meta data for all 
-        results. 
-        """
-        obj = self._connection.documents.get(id=self.id)
-        self.__dict__['contributor'] = obj.contributor
-        self.__dict__['contributor_organization'] = obj.contributor_organization
-        self.__dict__['data'] = obj.data
-        self.__dict__['annotations'] = obj.__dict__['annotations']
-        self.__dict__['sections'] = obj.__dict__['sections']
-    
     def get_contributor(self):
         """
         Fetch the contributor field if it does not exist.
         """
         try:
-            return self.__dict__['contributor']
+            return self.__dict__[u'contributor']
         except KeyError:
-            self._lazy_load()
-            return self.__dict__['contributor']
+            obj = self._connection.documents.get(id=self.id)
+            self.__dict__[u'contributor'] = obj.contributor
+            return obj.contributor
     contributor = property(get_contributor)
     
     def get_contributor_organization(self):
@@ -559,47 +432,25 @@ class Document(BaseAPIObject):
         Fetch the contributor_organization field if it does not exist.
         """
         try:
-            return self.__dict__['contributor_organization']
+            return self.__dict__[u'contributor_organization']
         except KeyError:
-            self._lazy_load()
-            return self.__dict__['contributor_organization']
+            obj = self._connection.documents.get(id=self.id)
+            self.__dict__[u'contributor_organization'] = obj.contributor_organization
+            return obj.contributor_organization
     contributor_organization = property(get_contributor_organization)
     
-    def set_data(self, data):
-        """
-        Update the data attribute, making sure it's a dictionary.
-        """
-        # Make sure a dict got passed it
-        if type(data) != type({}):
-            raise TypeError("This attribute must be a dictionary.")
-        # Validate keywords
-        for keyword in data.keys():
-            is_valid_data_keyword(keyword)
-        # Set the attribute
-        self.__dict__['data'] = data
-    
-    def get_data(self):
-        """
-        Fetch the data field if it does not exist.
-        """
-        try:
-            return self.__dict__['data']
-        except KeyError:
-            self._lazy_load()
-            return self.__dict__['data']
-    data = property(get_data, set_data)
-    
     def get_annotations(self):
         """
         Fetch the annotations field if it does not exist.
         """
         try:
-            obj_list = self.__dict__['annotations']
+            obj_list = self.__dict__[u'annotations']
             return [Annotation(i) for i in obj_list]
         except KeyError:
-            self._lazy_load()
-            obj_list = self.__dict__['annotations']
-            return [Annotation(i) for i in obj_list]
+            obj = self._connection.documents.get(id=self.id)
+            obj_list = [Annotation(i) for i in obj.__dict__['annotations']]
+            self.__dict__[u'annotations'] =obj.__dict__['annotations']
+            return obj_list
     annotations = property(get_annotations)
     
     def get_sections(self):
@@ -607,12 +458,13 @@ class Document(BaseAPIObject):
         Fetch the sections field if it does not exist.
         """
         try:
-            obj_list = self.__dict__['sections']
+            obj_list = self.__dict__[u'sections']
             return [Section(i) for i in obj_list]
         except KeyError:
-            self._lazy_load()
-            obj_list = self.__dict__['sections']
-            return [Section(i) for i in obj_list]
+            obj = self._connection.documents.get(id=self.id)
+            obj_list = [Section(i) for i in obj.__dict__['sections']]
+            self.__dict__[u'sections'] = obj.__dict__['sections']
+            return obj_list
     sections = property(get_sections)
     
     def get_entities(self):
@@ -620,39 +472,23 @@ class Document(BaseAPIObject):
         Fetch the entities extracted from this document by OpenCalais.
         """
         try:
-            return self.__dict__['entities']
+            return self.__dict__[u'entities']
         except KeyError:
-            entities = self._connection.fetch(
-                    "documents/%s/entities.json" % self.id
-                ).get("entities")
+            data = self._connection.fetch("documents/%s/entities.json" % self.id).get("entities")
             obj_list = []
-            for type, entity_list in entities.items():
+            for type, entity_list in data.items():
                 for entity in entity_list:
                     entity['type'] = type
                     obj = Entity(entity)
                     obj_list.append(obj)
-            self.__dict__['entities'] = obj_list
-            return self.__dict__['entities']
+            self.__dict__[u'entities'] = obj_list
+            return obj_list
     entities = property(get_entities)
     
     #
     # Text
     #
     
-    def _get_url(self, url):
-        if self.access == 'public':
-            req = urllib2.Request(url)
-            try:
-                return urllib2.urlopen(req).read()
-            except urllib2.HTTPError:
-                raise NotImplementedError(
-                    "Currently, DocumentCloud only allows you to access this resource on public documents."
-                )
-        else:
-            raise NotImplementedError(
-                "Currently, DocumentCloud only allows you to access this resource on public documents."
-            )
-    
     def get_full_text_url(self):
         """
         Returns the URL that contains the full text of the document.
@@ -664,7 +500,9 @@ class Document(BaseAPIObject):
         """
         Downloads and returns the full text of the document.
         """
-        return self._get_url(self.full_text_url)
+        req = urllib2.Request(self.full_text_url)
+        response = urllib2.urlopen(req)
+        return response.read()
     full_text = property(get_full_text)
     
     def get_page_text_url(self, page):
@@ -673,14 +511,16 @@ class Document(BaseAPIObject):
         """
         template = self.resources.page.get('text')
         url = template.replace("{page}", str(page))
-        return self._get_url(url)
+        return url
     
     def get_page_text(self, page):
         """
         Downloads and returns the full text of a particular page in the document.
         """
         url = self.get_page_text_url(page)
-        return self._get_url(url)
+        req = urllib2.Request(url)
+        response = urllib2.urlopen(req)
+        return response.read()
     
     #
     # Images
@@ -697,7 +537,9 @@ class Document(BaseAPIObject):
         """
         Downloads and returns the full PDF of the document.
         """
-        return self._get_url(self.pdf_url)
+        req = urllib2.Request(self.pdf_url)
+        response = urllib2.urlopen(req)
+        return response.read()
     pdf = property(get_pdf)
     
     def get_small_image_url(self, page=1):
@@ -707,7 +549,8 @@ class Document(BaseAPIObject):
         The page kwarg specifies which page to return. One is the default.
         """
         template = self.resources.page.get('image')
-        return template.replace("{page}", str(page)).replace("{size}", "small")
+        url = template.replace("{page}", str(page)).replace("{size}", "small")
+        return url
     small_image_url = property(get_small_image_url)
     
     def get_thumbnail_image_url(self, page=1):
@@ -717,7 +560,8 @@ class Document(BaseAPIObject):
         The page kwarg specifies which page to return. One is the default.
         """
         template = self.resources.page.get('image')
-        return template.replace("{page}", str(page)).replace("{size}", "thumbnail")
+        url = template.replace("{page}", str(page)).replace("{size}", "thumbnail")
+        return url
     thumbnail_image_url = property(get_thumbnail_image_url)
     
     def get_normal_image_url(self, page=1):
@@ -727,7 +571,8 @@ class Document(BaseAPIObject):
         The page kwarg specifies which page to return. One is the default.
         """
         template = self.resources.page.get('image')
-        return template.replace("{page}", str(page)).replace("{size}", "normal")
+        url = template.replace("{page}", str(page)).replace("{size}", "normal")
+        return url
     normal_image_url = property(get_normal_image_url)
 
     def get_large_image_url(self, page=1):
@@ -737,7 +582,8 @@ class Document(BaseAPIObject):
         The page kwarg specifies which page to return. One is the default.
         """
         template = self.resources.page.get('image')
-        return template.replace("{page}", str(page)).replace("{size}", "large")
+        url = template.replace("{page}", str(page)).replace("{size}", "large")
+        return url
     large_image_url = property(get_large_image_url)
     
     def get_small_image_url_list(self):
@@ -775,7 +621,9 @@ class Document(BaseAPIObject):
         The page kwarg specifies which page to return. One is the default.
         """
         url = self.get_small_image_url(page=page)
-        return self._get_url(url)
+        req = urllib2.Request(url)
+        response = urllib2.urlopen(req)
+        return response.read()
     small_image = property(get_small_image)
     
     def get_thumbnail_image(self, page=1):
@@ -785,7 +633,9 @@ class Document(BaseAPIObject):
         The page kwarg specifies which page to return. One is the default.
         """
         url = self.get_thumbnail_image_url(page=page)
-        return self._get_url(url)
+        req = urllib2.Request(url)
+        response = urllib2.urlopen(req)
+        return response.read()
     thumbnail_image = property(get_thumbnail_image)
     
     def get_normal_image(self, page=1):
@@ -795,7 +645,9 @@ class Document(BaseAPIObject):
         The page kwarg specifies which page to return. One is the default.
         """
         url = self.get_normal_image_url(page=page)
-        return self._get_url(url)
+        req = urllib2.Request(url)
+        response = urllib2.urlopen(req)
+        return response.read()
     normal_image = property(get_normal_image)
     
     def get_large_image(self, page=1):
@@ -805,7 +657,9 @@ class Document(BaseAPIObject):
         The page kwarg specifies which page to return. One is the default.
         """
         url = self.get_large_image_url(page=page)
-        return self._get_url(url)
+        req = urllib2.Request(url)
+        response = urllib2.urlopen(req)
+        return response.read()
     large_image = property(get_large_image)
     
     #
@@ -843,7 +697,7 @@ class Document(BaseAPIObject):
 
 class DocumentSet(list):
     """
-    A custom class for document lists associated with projects.
+    A custom class for project lists associated with projects.
     
     Allows some tweaks, like preventing duplicate documents
     from getting into the list and ensuring that only Document
@@ -893,7 +747,9 @@ class Mention(BaseAPIObject):
     A mention of a search found in the document.
     """
     def __unicode__(self):
-        return unicode("Page %s" % (self.page))
+        return unicode("%s: %s" % (self.page, self.text[:50]))
+
+
 
 
 class Project(BaseAPIObject):
@@ -907,9 +763,9 @@ class Project(BaseAPIObject):
         # Allow document_list to be zeroed out with [] or None
         if attr == 'document_list':
             if value == None:
-                self.__dict__['document_list'] = DocumentSet([])
+                self.__dict__[u'document_list'] = DocumentSet([])
             elif isinstance(value, list):
-                self.__dict__['document_list'] = DocumentSet(value)
+                self.__dict__[u'document_list'] = DocumentSet(value)
             else:
                 raise TypeError
         else:
@@ -933,18 +789,12 @@ class Project(BaseAPIObject):
         Returns nothing.
         """
         params = dict(
-            title=self.title or '',
-            description=self.description or '',
+            title=self.title,
+            description=self.description,
             document_ids=[str(i.id) for i in self.document_list]
         )
         self._connection.put('projects/%s.json' % self.id, params)
     
-    def save(self):
-        """
-        An alias to `'put` that post changes back to DocumentCloud
-        """
-        self.put()
-    
     def delete(self):
         """
         Deletes this object from documentcloud.org.
@@ -960,10 +810,10 @@ class Project(BaseAPIObject):
         Retrieves all documents included in this project.
         """
         try:
-            return self.__dict__['document_list']
+            return self.__dict__[u'document_list']
         except KeyError:
             obj_list = DocumentSet([self._connection.documents.get(i) for i in self.document_ids])
-            self.__dict__['document_list'] = obj_list
+            self.__dict__[u'document_list'] = obj_list
             return obj_list
     document_list = property(get_document_list)
     
@@ -1009,25 +859,5 @@ class Section(BaseAPIObject):
     pass
 
 
-RESERVED_KEYWORDS = [
-    'person', 'organization', 'place', 'term', 'email', 'phone',
-    'city', 'state', 'country', 'title', 'description', 'source',
-    'account', 'group', 'project', 'projectid', 'document', 'access',
-    'filter',
-]
 
-def is_valid_data_keyword(keyword):
-    """
-    Accepts a keyword submitted to the Document's "data" attribute and verifies
-    that is is safe and not one of the reserved words DocumentCloud does not
-    allow. 
-    
-    Returns True is keyword is safe. Raises ValueError if it is not.
-    
-    See: https://github.com/documentcloud/documentcloud/blob/master/config/initializers/entity_map.rb#L22
-    And: https://github.com/datadesk/python-documentcloud/issues/81
-    """
-    if keyword in RESERVED_KEYWORDS:
-        raise ValueError("The key %s is reserved by DocumentCloud. You can't use it in 'data'" % keyword)
-    else:
-        return True
+
